/*
 *
 * Copyright (c) 1996 - 1998
 * Frank Buss (fb@zuse.com), Stephan Schloepke (stephan@nordrhein.net)
 *
 * This applet was published in the book:
 *
 *     "Programmier Training Java, In 15 Stunden topfit"
 *     ISBN 3-8158-1303-4
 *     Authors: Frank Buss, Stephan Schloepke
 *     Data Becker Verlage (http://www.databecker.de)
 *
 * You can get this applet and more at http://www.zuse.com
 *
 */

//
// Das Spiel TicTacToe
//

import java.applet.*;
import java.awt.*;

// Klasse fr ein Bild als Komponente einen Container-Objekt hinzuzufgen
class BildKomponente extends Canvas {

    private Image image;  // das anzuzeigende Bild

    // Konstruktor zum setzen der Klassenvariable und der Gre des Canvas-Objekts
    // auf die Bildgre
    public BildKomponente(Image image) {
        this.image=image;
        resize(image.getWidth(this),image.getHeight(this));
    }

    // anzeigen des Bildes
    public void paint(Graphics g) {
        g.drawImage(image,0,0,this);
    }

    // update berschreiben, da der Zeichenbereich nicht gelscht werden braucht,
    // was die Standard-Implementierung der update-Methode ist
    public void update(Graphics g) {
        paint(g);
    }
}

// die Ableitung der Applet-Klasse
// sie fungiert als Koordinator zwischen den Buttons, Labels und dem Spielfeld
public class TicTacToe extends Applet {

    // folgende Variablen beinhalten Referenzen auf Grafikelemente
    private Spielfeld Feld;  // das Spielfeld
    private Button Ruecksetz;  // der Button mit dem Text "Punktestand zurcksetzen"
    private Label Gew1Lbl;  // Text fr Anzahl gewonnener Runden fr Spieler 1
    private Label Gew2Lbl;  // fr Spieler 2

    // Variablen zur Zhlung der Anzahl Gewinne fr die beiden Spieler
    private int Gew1=0;
    private int Gew2=0;

    // Spielfeld leeren und neu zeichen
    public void Neu() {
        Feld.init();
        Feld.repaint();
    }

    // Bild holen und warten, bis es geladen ist
    private Image ladeBild(String bild) {

        // Bild holen
        Image image=getImage(getDocumentBase(),bild);

        // ein MediaTracker-Objekt anlegen und das Bild davon kontrollieren lassen
        MediaTracker tracker=new MediaTracker(this);
        tracker.addImage(image,0);

        // solange warten, bis das Bild vollstndig geladen ist
        try {
            tracker.waitForID(0);
        } catch (InterruptedException e) { }

        // Referenz auf das Bild zurckliefern
        return image;
    }

    // den Gewinntext fr die Anzahl gewonnener Runden beider Spieler aktualisieren
    public void aktuGewinntext() {
        String GewinnText="Gewonnene Runden: ";
        Gew1Lbl.setText(GewinnText+Gew1);
        Gew2Lbl.setText(GewinnText+Gew2);
    }

    // Methode zum erhhen des Punktestandes fr einen Spieler und neu starten des Spiels
    // das Argument "spieler" kann auch 0 sein, falls die Runde unentschieden ausging
    public void Gewonnen(int spieler) {
        switch(spieler) {
            case 1:
                Gew2++;
                break;
            case 2:
                Gew1++;
                break;
        }
        aktuGewinntext();
        Neu();
    }

    // Applet initialisieren
    public void init() {

        setBackground(Color.white);
        
        // die beiden Grafiken fr die Spielersymbole laden
        Image kreuz=ladeBild("kreuz.gif");
        Image kreis=ladeBild("kreis.gif");

        // neue Schrift whlen
        setFont(new Font("TimesRoman",Font.PLAIN,16));

        // Layout fr das Applet ndern
        setLayout(new BorderLayout());

        // Label fr Anzahl gewonnener Runden erzeugen und aktualisieren
        Gew1Lbl=new Label();
        Gew2Lbl=new Label();
        aktuGewinntext();        

        // Spielername, Grafiksymbol und Anzahl Gewinne von Spieler 1 dem Applet hinzufgen
        Panel Spieler1=new Panel();
        Spieler1.add(new Label("Spieler 1"));
        Panel links=new Panel();
        links.setLayout(new BorderLayout());
        links.add("North",Spieler1);
        Panel LiMi=new Panel();
        LiMi.add("Center",new BildKomponente(kreuz));
        links.add("Center",LiMi);
        links.add("South",Gew1Lbl);
        add("West",links);

        // Spielername, Grafiksymbol und Anzahl Gewinne von Spieler 2 dem Applet hinzufgen
        Panel Spieler2=new Panel();
        Spieler2.add(new Label("Spieler 2"));
        Panel rechts=new Panel();
        rechts.setLayout(new BorderLayout());
        rechts.add("North",Spieler2);
        Panel ReMi=new Panel();
        ReMi.add("Center",new BildKomponente(kreis));
        rechts.add("Center",ReMi);
        rechts.add("South",Gew2Lbl);
        add("East",rechts);

        // die Toolbar erzeugen und im Norden des Applets positionieren
        Panel leiste=new Panel();
        Ruecksetz=new Button("Punktestand zurcksetzen");
        leiste.add(Ruecksetz);
        add("North",leiste);

        // das Spielfeld auf einem Panel plaziert in dem Applet zentriert anzeigen
        Panel mitte=new Panel();
        Feld=new Spielfeld(this,kreuz,kreis);
        Feld.init();
        mitte.add(Feld);
        add("Center",mitte);
    }

    // testet die Schaltflche "Punktestand zurcksetzen"
    public boolean handleEvent(Event evt) {

        // nur nher testen, wenn die Nachricht ACTION_EVENT aufgetreten war
        if (evt.id==Event.ACTION_EVENT) {

            // bei "Punktestand zurcksetzen" Zhler auf 0 setzen und aktualisieren
            if (evt.target==Ruecksetz) {
                Gew1=0;
                Gew2=0;
                aktuGewinntext();
            }
        }

        // Status der Nachrichten-Behandlungs-Methode der Basisklasse zurckliefern
        return super.handleEvent(evt);
    }

}
